#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
* primer dato designa : nota off(80,128), nota on(90,144),
    controller change(B0,176))
* segundo dato designa la nota que queremos encender.Ver esquema launchpad
    Hex version     -- Key = (10h x Row) + Column
    Decimal version -- Key = ( 16 x Row) + Column

    En hex: 00-08, 10-18,20-28,30-38,40-48,50-58,60-68,70-78
    En Dec: 0-8, 16-24,32-40,48-56,64-72,80-88,96-104,(104-111),112-120

* tercer dato designa el color, en los manuales es la velocidad
    The following tables of pre-calculated velocity values for normal use may
    also be helpful:
        Hex Decimal Colour Brightness
        0Ch 12 Off Off
        0Dh 13 Red Low
        0Eh 14 Red Medium
        0Fh 15 Red Full
        1Dh 29 Amber Low
        3Fh 63 Amber Full
        3Eh 62 Yellow Full
        1Ch 28 Green Low
        3Ch 60 Green Full

    Values for flashing LEDs are:
        Hex Decimal Colour Brightness
        0Bh 11 Red Full
        3Bh 59 Amber Full
        3Ah 58 Yellow Full
        38h 56 Green Full

    Devices:
    return (interf, name, input, output, opened)

pygame-> midi
"""

import alsaaudio as alsa
import sys
import threading
import operator
import pygame.midi
import time

pygame.midi.init()

devices1 = pygame.midi.get_device_info(2)
devices2 = pygame.midi.get_device_info(3)

launchpad_o = 2
launchpad_i = 3


class Launchpad:
    """
    Descripcion
    """
    def __init__(self):
        self.midi_in = pygame.midi.Input(launchpad_i)
        self.midi_out = pygame.midi.Output(launchpad_o)
        for posicion in range(0x00, 0x120):
            self.midi_out.note_off(posicion)
        self.active = True

    def Start(self):
        self.midi_out.write_short(176, 111, 15)

    def Activate(self):
        self.active = True

    def Desactivate(self):
        self.active = False

    def Get_Init(self):
        while 1 != 0:
            dato = self.midi_in.poll()
            if dato:
                read = self.midi_in.read(1)
                key = read[0][0][1]
                on = read[0][0][2]
                if (key == 111)and (on == 127):
                    self.midi_out.write_short(176, 111, 60)
                    self.Logo_On()
                    self.Activate()
                    return True
                    break

    def Get_Out(self):
        self.Keys_Off()
        self.Desactivate()
        self.midi_out.write_short(176, 111, 15)
        self.Logo_Off()

    def Reader(self):
        while 1 != 0:
            dato = self.midi_in.poll()
            if dato:
                read = self.midi_in.read(1)
                key = read[0][0][1]
                 #Control de tecla pulsada, y no mantenida
                if read[0][0][2] == 127:
                    return key
                    break

    def Writer(self, key, color):
        if self.active:
            self.midi_out.write_short(144, key, color)

    def Writer_cname(self, key, colorname):
        if self.active:
            if colorname == 'red':
                color = 15
            if colorname == 'amber':
                color = 63
            if colorname == 'green':
                color = 60
            self.midi_out.write_short(144, key, color)

    def Writer_mod(self, mode, key, color):
        if self.active:
            self.midi_out.write_short(mode, key, color)

    def Writer_mod_cname(self, mode, key, colorname):
        if self.active:
            if colorname == 'red':
                color = 15
            if colorname == 'amber':
                color = 63
            if colorname == 'green':
                color = 60
            self.midi_out.write_short(mode, key, color)

    def Flicker_lock(self, key, color):
        if self.active:
            while 1 != 0:
                self.midi_out.write_short(144, key, color)
                time.sleep(0.5)
                self.midi_out.write_short(144, key, 12)
                time.sleep(0.2)

    def Flicker(self, key, color):
        if self.active:
            flick = threading.Thread(target=self.Flicker_lock,
                                    args=(key, color, ))
            flick.start()

    def Set_Fader(self, channel, value):
        if self.active:
            value = 80 - (16 * (value / 20))
            for posicion in range(0, 120):
                if operator.mod(posicion, 8) == (channel - 1)\
                    and posicion >= value and posicion <= 88:
                    self.midi_out.write_short(144, posicion, 60)
                elif operator.mod(posicion, 8) == (channel - 1)\
                    and posicion < value:
                    self.midi_out.note_off(posicion)

    def Get_Fader(self):
        resultado = False
        while not resultado:
            dato = self.midi_in.poll()
            if dato:
                read = self.midi_in.read(1)
                posicion = read[0][0][1]
                if posicion != 104:
                    channel = (operator.mod(posicion, 8)) + 1
                    if posicion == 0:
                        value = 100
                    else:
                        value = 100 - (20 * (posicion / 16))
                    if value <= 88:
                        return channel, value
                        return True

    def Midi_paint(self, channel, volume_inicial, color):
        if self.active:
            if color == 'green':
                key_color = 60
            elif color == 'amber':
                key_color = 63
            elif color == 'red':
                key_color = 15

            if volume_inicial == 0:
                row = 0
            elif volume_inicial <= 19:
                row = 1
            elif volume_inicial <= 39:
                row = 2
            elif volume_inicial <= 61:
                row = 3
            elif volume_inicial <= 71:
                row = 4
            elif volume_inicial <= 81:
                row = 5
            elif volume_inicial <= 90:
                row = 6
            elif volume_inicial < 101:
                row = 7

            for i in range(0, 8):
                key = (16 * (7 - i) + (channel - 1))
                if i <= row:
                    self.Writer(key, key_color)
                else:
                    self.Writer(key, 12)

    def Paint(self):
        if self.active:
            color = 12
            while 1 != 0:
                dato = self.midi_in.poll()
                if dato:
                    read = self.midi_in.read(1)
                    key = read[0][0][1]
                    if key == 104:
                        color = 15
                    if key == 105:
                        color = 63
                    if key == 106:
                        color = 62
                    if key == 107:
                        color = 60
                    if key == 108:
                        for i in range(0x00, 0x120):
                            self.midi_out.note_off(i)
                    if key == 109:
                        color = 12
                    if key == 111:
                        break
                    if read[0][0][2] == 127:
                        self.midi_out.write_short(144, key, color)

    def Color_Calibrate(self):
        if self.active:
            for num in [12, 13, 15, 29, 63, 62, 60, 12]:
                for key in range(0, 120):
                    self.midi_out.write_short(144, key, num)
                    time.sleep(0.001)

    def Keys_Off(self):
        for i in range(0, 120):
            self.midi_out.note_off(i)

    def Logo_On(self):
        for key in [3, 4, 19, 20, 35, 36, 51, 52, 67, 68, 87, 103, 71,
                    55, 23, 39, 38, 22, 17, 33, 16, 32, 48, 64, 80, 96,
                    112, 113, 114, 115, 116, 117, 118, 119]:
            self.midi_out.write_short(144, key, 60)
        time.sleep(2)
        self.Keys_Off()

    def Logo_Off(self):
        for key in [3, 4, 19, 20, 35, 36, 51, 52, 67, 68, 87, 103, 71,
                    55, 23, 39, 38, 22, 17, 33, 16, 32, 48, 64, 80, 96,
                    112, 113, 114, 115, 116, 117, 118, 119]:
            self.midi_out.write_short(144, key, 15)
        time.sleep(2)
        self.Keys_Off()

if __name__ == '__main__':

    launchpad = Launchpad()
    print "Modos" + '\n' + "1.-Apagar las teclas" + '\n' +\
            "2.-Calibracion de color" + '\n' + "3.-Paint" +\
            '\n' + "4.-Devices" + '\n' + "5.-Salir"

    while 1 != 0:
        modo = int(input("Modo: "))
        if modo == 1:
            launchpad.Keys_Off()
        if modo == 2:
            launchpad.Color_Calibrate()
        if modo == 3:
            launchpad.Paint()
        if modo == 4:
            print "Devices" + "\n" + str(devices1) + "\n" + str(devices2)
        if modo == 5:
            break
