#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
Control del sonido a traves de los diferentes Channeles que
nos permite trabajar una tarjeta 5.1 estandar, obteniendo
3 Channeles estereo (+1 LFE), o 6 Channeles mono.

Niveles soportados 0,3,6,10,13,16,19,23,26,29,32,35,39,42,
45,48,52,55,58,61,65,68,71,74,77,81,84,87,90,94,97,100
Valores diferentes a estos redondea.

Tarjeta de sonido por defecto 0.

pyalsaaudio
"""

import alsaaudio as alsa
import os
import sys
import threading
import time

"""
MIXER
"""


class Mixer:

    def __init__(self, soundcard, numchannel):
        """
        Inicializa la tarjeta de sonido, y los diferentes canales.
        """
        self.souncard = soundcard
        self.numchannel = numchannel
        self.Master = alsa.Mixer('Master', soundcard)
        self.Channel = []
        if self.numchannel == 1:
            mixers = alsa.mixers(0)
            for channel in mixers:
                if channel == 'Front':
                    self.Channel.append(alsa.Mixer('Front', soundcard))
                elif channel == 'Speaker':
                    self.Channel.append(alsa.Mixer('Speaker', soundcard))
        if self.numchannel == 2:
            self.Channel.append(alsa.Mixer('Front', soundcard))
            self.Channel.append(alsa.Mixer('Side', soundcard))
        if self.numchannel == 3:
            self.Channel.append(alsa.Mixer('Front', soundcard))
            self.Channel.append(alsa.Mixer('Side', soundcard))
            self.Channel.append(alsa.Mixer('Surround', soundcard))

    def SoundChannels(self):
        """
        Este metodo establece el valor maximo de volumen
        para todos los canales.
        """
        if self.numchannel == 1:
            self.Channel[0].setvolume(100)
        if self.numchannel == 2:
            self.Channel[0].setvolume(100)
            self.Channel[1].setvolume(100)
        if self.numchannel == 3:
            self.Channel[0].setvolume(100)
            self.Channel[1].setvolume(100)
            self.Channel[2].setvolume(100)

    def ShutUpVolume(self):
        """
        Este metodo establece el valor minimo de volumen
        para todos los canales.
        """
        if self.numchannel == 1:
            self.Channel[0].setvolume(0)
        if self.numchannel == 2:
            self.Channel[0].setvolume(0)
            self.Channel[1].setvolume(0)
        if self.numchannel == 3:
            self.Channel[0].setvolume(0)
            self.Channel[1].setvolume(0)
            self.Channel[2].setvolume(0)

    def SoundChannel(self, channel):
        """
        Este metodo establece el valor maximo de volumen
        para un canal especifico.
        """
        self.Channel[channel - 1].setvolume(100)

    def ShutUpChannel(self, channel):
        """
        Este metodo establece el valor minimo de volumen
        para un canal especifico.
        """
        self.Channel[channel - 1].setvolume(0)

    def TurnOffChannel(self, channel):
        """
        Este metodo realiza un fadeout de canal especificado.
        """
        volume = self.Channel[channel - 1].getvolume()
        for i in range(1, (volume[0] / 10)):
            if volume[0] != 0:
                volume[0] = volume[0] - 10
                self.Channel[channel - 1].setvolume(volume[0])
                time.sleep(0.1)

    def SetVolume(self, channel, volume):
        """
        Este metodo establece el valor introducido por parametro de volumen
        para un canal especifico.
        """
        self.Channel[channel - 1].setvolume(volume)

    def GetVolume(self, channel):
        """
        Este metodo nos devuelve el valor de volumen de un canal especifico.
        """
        return self.Channel[channel - 1].getvolume()

    def SetMaster(self, volume):
        """
        Este metodo establece el valor de volumen para el canal 'Master'
        """
        self.Master.setvolume(volume)

    def GetMaster(self):
        """
        Este metodo nos devuelve el valor de volumen del canal 'Master'
        """
        return self.Master.getvolume()

    def Fader(self, Volume):
        """
        Este metodo nos muestra graficamente en la shell el valor
        introducido por parametro.
        """
        Volume_der = Volume[0] / 5
        Volume_izq = Volume[1] / 5
        linea_der = '\033[32;1m' + '◾'
        linea_izq = '\033[32;1m' + '◾'
        for i in range(1, Volume_der):
            if i <= 10:
                linea_der += '\033[32;1m'
            if i > 10:
                linea_der += '\033[33;1m'
            if i > 15:
                linea_der += '\033[31;1m'
            linea_der += '◾' + '\033[0m'
        for i in range(1, Volume_izq):
            if i <= 10:
                linea_izq += '\033[32;1m'
            if i > 10:
                linea_izq += '\033[33;1m'
            if i > 15:
                linea_izq += '\033[31;1m'
            linea_izq += '◾' + '\033[0m'
        print "Canal der: " + linea_der + "  " + str(Volume_der * 5) +\
             '\n' + '\033[30;0m' + "Canal izq: " + linea_izq + "  " +\
             str(Volume_izq * 5) + '\033[30;0m'

    def DisplayVolumes(self):
        """
        Este metodo parte de metodo anterior y muestra graficamente el
        valor de volumen de los diferentes canales inicializados.
        """
        for i in range(0, self.numchannel):
            self.Fader(self.Channel[i].getvolume())

    def DisplayVolume(self, channel):
        """
        Este metodo parte de metodo anterior y muestra graficamente el
        valor de volumen de un canal especifico.
        """
        self.Fader(self.Channel[channel].getvolume())


"""
Programa principal
"""
if __name__ == '__main__':
    if len(sys.argv) != 3:
        print chr(27) + "[3;31m" + \
        "Usage: Tarjeta de sonido  Modo reproductor" + chr(27) + "[0m"
        sys.exit()
    else:
        mezclador = Mixer(int(sys.argv[1]), int(sys.argv[2]))
        print '0 - Shut up' + '\n' +\
              '1 - Sound channel 1' + '\n' +\
              '2 - Sound channel 2' + '\n' +\
              '3 - Sound channel 3' + '\n' +\
              '4 - Choose sound channel' + '\n' +\
              '9 - Exit' + '\n'
        try:
            while 1 != 0:
                mezclador.DisplayVolumes()
                modo = int(input("Modo reproductor: "))
                if modo == 0:
                    mezclador.ShutUpVolume()
                if modo == 1:
                    mezclador.ShutUpVolume()
                    mezclador.SoundChannel(1)
                if modo == 2:
                    mezclador.ShutUpVolume()
                    mezclador.SoundChannel(2)
                if modo == 3:
                    mezclador.ShutUpVolume()
                    mezclador.SoundChannel(3)
                if modo == 4:
                    canal = int(input("Canal reproductor: "))
                    volumen = int(input("Volumen: "))
                    mezclador.SetVolume(canal, volumen)
                if modo == 9:
                    print "Salir"
                    sys.exit()

        except KeyboardInterrupt:
            print 'Bye.'
