#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
En este documento se encuentra la clase player,la cual nos permite la
reproduccion de archivos mp3, asi como crear una lista de reproduccion.
Dispone de 8 metodos, los cuales nos permiten hacer play, pause, stop,
next, previous y crear un end listener.
"""

import os
import sys
import time
import threading
import pygame
import glob


class Player(threading.Thread):

    def __init__(self):
        """
        Inicializa el reproductor con los parametros de reproduccion,
        asi como guardarse la lista de canciones reproducibles.
        """
        pygame.mixer.pre_init(44100, -16, 2, 4096)
        pygame.init()

        threading.Thread.__init__(self)
        self.cancion = pygame.mixer.music
        self.SONG_END = pygame.USEREVENT
        self.cancion.set_endevent(self.SONG_END)

        self.path_files = "Sources"
        os.chdir(self.path_files)
        self.music_list = glob.glob("*.mp3")
        self.song_counter = 0
        self.is_playing = False

    def list_files(self):
        """
        Muestra la lista de canciones
        """
        print self.music_list

    def run(self):
        """
        Este metodo comprueba la existencia de canciones y carga la primera de
        la lista
        """
        list_songs = os.listdir(".")
        if len(list_songs) > 0:
            self.cancion.load(self.music_list[self.song_counter])
        else:
            print "Warning: No music available."

    def play(self):
        """
        Este metodo permite reproducir la cancion cargada
        """
        self.cancion.play(0)
        self.is_playing = True

    def pause(self):
        """
        Este metodo permite poder pausar y reanudar una cancion.
        """
        if self.is_playing:
            self.cancion.pause()
        else:
            self.cancion.unpause()
        self.is_playing = not self.is_playing

    def next(self):
        """
        Este metodo permite pasar a la cancion siguiente
        """
        self.cancion.pause()
        self.is_playing = not self.is_playing
        if (len(self.music_list) - 1) != self.song_counter:
            self.song_counter += 1
        else:
            self.song_counter = 0
        self.cancion.load(self.music_list[self.song_counter])
        self.cancion.play(0)

    def previous(self):
        """
        Este metodo permite pasar a la cancion anterior.
        """
        self.cancion.pause()
        self.is_playing = not self.is_playing
        if self.song_counter > 0:
            self.song_counter -= 1
        self.cancion.load(self.music_list[self.song_counter])
        self.cancion.play(0)

    def stop(self):
        """
        Este metodo permite parar la cancion y reiniciar la lista de
        reproduccion.
        """
        self.cancion.pause()
        self.is_playing = not self.is_playing
        self.song_counter = 0
        self.cancion.load(self.music_list[self.song_counter])

    def song_end(self):
        """
        Este metodo permite la reproduccion consecutiva de las canciones
        de la lista de reproduccion, y cuando termina la lista vuelve al
        principio y para la reproduccion.
        """
        while True:
            for event in pygame.event.get():
                if event.type == self.SONG_END:
                    if (len(self.music_list) - 1) != self.song_counter:
                        self.song_counter += 1
                        self.cancion.load(self.music_list[self.song_counter])
                        self.cancion.play(0)
                        break
                    else:
                        self.song_counter = 0
                        self.cancion.load(self.music_list[self.song_counter])
                        break

    def play_list(self):
        """
        Este metodo permite crear un hilo para el detector de fin de cancion.
        """
        play_list = threading.Thread(target=self.song_end, args=())
        play_list.start()

"""
Programa principal especifico
"""
if __name__ == '__main__':
    while 1 != 0:
        cancion = Player()
        cancion.start()
        cancion.play_list()
        print "1.play" + '\n' + "2.pause" + '\n' + "3.stop(fade out)" +\
                '\n' + "4.seleccionar cancion" + '\n' + "5.exit" +\
                '\n' + "7.siguiente" + '\n' + "8.anterior" + '\n'
        while pygame.mixer.music.get_busy:
            print '>>'
            function = int(input(""))
            if function == 1:
                cancion.play()
            if function == 2:
                cancion.pause()
            if function == 3:
                cancion.stop()
            if function == 4:
                cancion.list_files()
                filename = raw_input("Archivo: ")
                print filename
                break
            if function == 5:
                sys.exit()
            if function == 7:
                cancion.next()
            if function == 8:
                cancion.previous()
