#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
Scanner, programa que detecta la presencia del usuario a traves de una webcam,
capturandola y comparandola con la imagen que tiene guardado en memoria y
viendo su diferencia se comprueba si existe un cambio para detectar la
presencia o no.

calibracion dinamica -> error captura de imagenes

opencv python
v4l -v4l2 diversos codecs de video
webcam lib
libdc1394
libhighgui2.3
libopencv-contrib2.3
"""

import time
import sys
import cv
import threading


class Scanner:

    def __init__(self, numcam):
        self.numcam = numcam - 1
        self.photo_calibrate = cv.CreateImage((320, 240), 8, 3)
        self.imagediff = cv.CreateImage((320, 240), 8, 3)

    def Properties(self):
        """
        Este metodo nos permite saber las propiedades definidas en opencv que
        caracterizan las webcams conectadas.
        """
        capture = cv.CaptureFromCAM(self.numcam)
        print "**** 1 ****" +\
        str(cv.GetCaptureProperty(capture, cv.CV_CAP_PROP_POS_MSEC))
        print "**** 2 ****" +\
        str(cv.GetCaptureProperty(capture, cv.CV_CAP_PROP_POS_FRAMES))
        print "**** 3 ****" +\
        str(cv.GetCaptureProperty(capture, cv.CV_CAP_PROP_POS_AVI_RATIO))
        print "**** 4 ****" +\
        str(cv.GetCaptureProperty(capture, cv.CV_CAP_PROP_FRAME_WIDTH))
        print "**** 5 ****" +\
        str(cv.GetCaptureProperty(capture, cv.CV_CAP_PROP_FRAME_HEIGHT))
        print "**** 6 ****" +\
        str(cv.GetCaptureProperty(capture, cv.CV_CAP_PROP_FPS))
        print "**** 7 ****" +\
        str(cv.GetCaptureProperty(capture, cv.CV_CAP_PROP_FOURCC))
        print "**** 8 ****" +\
        str(cv.GetCaptureProperty(capture, cv.CV_CAP_PROP_FRAME_COUNT))
        print "**** 9 ****" +\
        str(cv.GetCaptureProperty(capture, cv.CV_CAP_PROP_FORMAT))
        print "**** 10 ****" +\
        str(cv.GetCaptureProperty(capture, cv.CV_CAP_PROP_MODE))
        print "**** 11 ****" +\
        str(cv.GetCaptureProperty(capture, cv.CV_CAP_PROP_BRIGHTNESS))
        print "**** 12 ****" +\
        str(cv.GetCaptureProperty(capture, cv.CV_CAP_PROP_CONTRAST))
        print "**** 13 ****" +\
        str(cv.GetCaptureProperty(capture, cv.CV_CAP_PROP_SATURATION))
        print "**** 14 ****" +\
        str(cv.GetCaptureProperty(capture, cv.CV_CAP_PROP_HUE))
        print "**** 15 ****" +\
        str(cv.GetCaptureProperty(capture, cv.CV_CAP_PROP_GAIN))
        print "**** 16 ****" +\
        str(cv.GetCaptureProperty(capture, cv.CV_CAP_PROP_EXPOSURE))
        print "**** 17 ****" +\
        str(cv.GetCaptureProperty(capture, cv.CV_CAP_PROP_CONVERT_RGB))
        print "**** 18 ****" +\
        str(cv.GetCaptureProperty(capture, cv.CV_CAP_PROP_RECTIFICATION))

    def ViewCam(self):
        """
        Funcion que muestra en tiempo real las
        imagenes obtenidas por la camara.
        """
        #Captura imagenes de la webcam
        capture = cv.CaptureFromCAM(self.numcam)
        #Fijamos el tamaño de las imagenes
        cv.SetCaptureProperty(capture, cv.CV_CAP_PROP_FRAME_WIDTH, 320)
        cv.SetCaptureProperty(capture, cv.CV_CAP_PROP_FRAME_HEIGHT, 240)
        #Creamos una nueva ventana de visualizacion
        cv.NamedWindow('View Cam ' + str(self.numcam))
        while True:
            #Trabajamos con un frame del video de la webcam
            frame = cv.QueryFrame(capture)
            #Mostramos el frame obtenido
            cv.ShowImage('View Cam ' + str(self.numcam), frame)
            k = cv.WaitKey(10)
            #Con la tecla escape salimos del visionado
            if k % 256 == 27:
                break
        cv.DestroyWindow('View Cam ' + str(self.numcam))

    def PhotoCalibrate(self):
        """
        Funcion que realiza una foto y la guarda en el fichero
        """
        capture = cv.CaptureFromCAM(self.numcam)
        cv.SetCaptureProperty(capture, cv.CV_CAP_PROP_FRAME_WIDTH, 320)
        cv.SetCaptureProperty(capture, cv.CV_CAP_PROP_FRAME_HEIGHT, 240)
        time_clock = 1
        while time_clock < 80:
            time_clock += 1
            frame = cv.QueryFrame(capture)
        cv.SaveImage("CALIBRATE_" + str(self.numcam) + ".jpg", frame)
        self.photo_calibrate = cv.LoadImage("CALIBRATE_"
                                            + str(self.numcam)
                                            + ".jpg")
        print "** Captura Realizada **"

    def ViewPhotoCalibrate(self):
        """
        Funcion que muestra las imagenes de calibracion que
        residen en el fichero.
        """
        frame = self.photo_calibrate
        cv.NamedWindow('Photo Calibrate ' + str(self.numcam))
        while True:
            cv.ShowImage('Photo Calibrate ' + str(self.numcam), frame)
            k = cv.WaitKey(10)
            if k % 256 == 27:
                break
        cv.DestroyWindow('Photo Calibrate ' + str(self.numcam))

    def Photo(self):
        """
        Funcion que realiza una captura, la guarda en el fichero,
        y la muestra
        """
        print "PHOTO ACTIVATE in cam: " + str(self.numcam)
        capture = cv.CaptureFromCAM(self.numcam)
        cv.SetCaptureProperty(capture, cv.CV_CAP_PROP_FRAME_WIDTH, 320)
        cv.SetCaptureProperty(capture, cv.CV_CAP_PROP_FRAME_HEIGHT, 240)
        frame = cv.QueryFrame(capture)
        cv.SaveImage("PHOTO_" + str(self.numcam) + ".jpg", frame)
        cv.NamedWindow('Photo ' + str(self.numcam))
        while True:
            cv.ShowImage('Photo ' + str(self.numcam), frame)
            k = cv.WaitKey(10)
            if k % 256 == 27:
                break
        cv.DestroyWindow('Photo ' + str(self.numcam))

    def Sub_RGB(self, mode=0):
        """
        Funcion que realiza la resta entre la imagen que tenemos guardada
        en el fichero con las imagenes que va obteniendo en tiempo real.
        """
        capture = cv.CaptureFromCAM(self.numcam)
        cv.SetCaptureProperty(capture, cv.CV_CAP_PROP_FRAME_WIDTH, 320)
        cv.SetCaptureProperty(capture, cv.CV_CAP_PROP_FRAME_HEIGHT, 240)
        cv.NamedWindow('Subtraction ' + str(self.numcam))
        i = 1
        while True:
            frame = cv.QueryFrame(capture)
            cv.Sub(frame, self.photo_calibrate, self.imagediff)
            cv.ShowImage('Subtraction ' + str(self.numcam), self.imagediff)
            Avg = cv.Avg(self.imagediff)
            if Avg[0] > 2.5 and Avg[1] > 2.5 and Avg[2] > 2.5:
                i += 1
                print "DETECTADO" + str(i)
                #return true
            k = cv.WaitKey(10)
            if k % 256 == 27:
                break
        cv.SaveImage("SUBTRACTION_"
                     + str(self.numcam)
                     + ".jpg",
                     self.imagediff)
        cv.DestroyWindow('Subtraction ' + str(self.numcam))

    def AppSub_RGB_init(self):
        capture = cv.CaptureFromCAM(self.numcam)
        cv.SetCaptureProperty(capture, cv.CV_CAP_PROP_FRAME_WIDTH, 320)
        cv.SetCaptureProperty(capture, cv.CV_CAP_PROP_FRAME_HEIGHT, 240)
        return capture

    def AppSub_RGB(self, capture):
        """
        Funcion que realiza la resta entre la imagen que tenemos guardada
        en el fichero con las imagenes que va obteniendo en tiempo real.
        """
        frame = cv.QueryFrame(capture)
        cv.Sub(frame, self.photo_calibrate, self.imagediff)
        Avg = cv.Avg(self.imagediff)
        if Avg[0] > 2.5 and Avg[1] > 2.5 and Avg[2] > 2.5:
            return True
        else:
            return False

    #probar con este restador aver si detecta mejor
    def Sub_HSV(self):

        frame_calibrate = cv.LoadImage("CALIBRATE_"
                                       + str(self.numcam)
                                       + ".jpg")
        frame_calibrate2 = cv.LoadImage("CALIBRATE_"
                                        + str(self.numcam)
                                        + ".jpg")
        cv.CvtColor(frame_calibrate, frame_calibrate2, cv.CV_RGB2HSV)

        capture = cv.CaptureFromCAM(self.numcam)
        cv.SetCaptureProperty(capture, cv.CV_CAP_PROP_FRAME_WIDTH, 320)
        cv.SetCaptureProperty(capture, cv.CV_CAP_PROP_FRAME_HEIGHT, 240)

        imagediff = cv.LoadImage("CALIBRATE_" + str(self.numcam) + ".jpg")
        imagediff2 = cv.CreateImage(cv.GetSize(imagediff), 8, 3)

        cv.NamedWindow('Subtraction ' + str(self.numcam))
        while True:
            frame = cv.QueryFrame(capture)
            frame2 = cv.CreateImage(cv.GetSize(frame), 8, 3)
            cv.CvtColor(frame, frame2, cv.CV_RGB2HSV)

            cv.Sub(frame2, frame_calibrate2, imagediff)
            cv.CvtColor(imagediff, imagediff2, cv.CV_HSV2RGB)

            cv.ShowImage('Subtraction ' + str(self.numcam), imagediff2)

            Avg = cv.Avg(imagediff)
            print "Cam: " + str(self.numcam)
            print Avg
            if Avg[0] > 2.5 and Avg[1] > 2.5 and Avg[2] > 2.5:
                print "DETECTADO ******************************"
            k = cv.WaitKey(10)
            if k % 256 == 27:
                break
        cv.SaveImage("SUBTRACTION_" + str(self.numcam) + ".jpg", imagediff)
        cv.DestroyWindow('Subtraction ' + str(self.numcam))

    def Sub_Dynamic(self, mode=0):
        """
        Calibracion dinamica
        La idea es, partimos de la foto de calibracion original, y cada minuto
        realiza una foto a traves de modo de transparencia, la cual sera la
        nueva foto de calibracion automatica (esta se realizara cuando no se
        detecte a nadie, a menos que se vuelva a realizar una calibracion
        manual.
        """
        frame_calibrate = cv.LoadImage("CALIBRATE_"
                                       + str(self.numcam)
                                       + ".jpg")
        capture = cv.CaptureFromCAM(self.numcam)
        cv.SetCaptureProperty(capture, cv.CV_CAP_PROP_FRAME_WIDTH, 320)
        cv.SetCaptureProperty(capture, cv.CV_CAP_PROP_FRAME_HEIGHT, 240)
        imagediff = cv.LoadImage("CALIBRATE_" + str(self.numcam) + ".jpg")
        imagediff2 = cv.LoadImage("CALIBRATE_" + str(self.numcam) + ".jpg")

        cv.NamedWindow('Subtraction ' + str(self.numcam))

        false_contador = 0.001
        true_contador = 0.001

        while True:
            frame = cv.QueryFrame(capture)

            cv.Sub(frame, frame_calibrate, imagediff)  # Resta
            cv.Sub(frame, imagediff, imagediff2)  # Transparencia mal hecha
            #cv.Sub(frame_calibrate, frame, imagediff2)  # Resta

            cv.ShowImage('Subtraction ' + str(self.numcam), imagediff)

            Avg = cv.Avg(imagediff)
            print "Cam: " + str(self.numcam)
            print Avg

            if Avg[0] > 2.5 and Avg[1] > 2.5 and Avg[2] > 2.5:
                true_contador += 0.001
                print "True: " + str(true_contador)
                if true_contador > 0.05:
                    cv.SaveImage("CALIBRATE_"
                                + str(self.numcam)
                                + ".jpg",
                                imagediff2)
                    frame_calibrate = cv.LoadImage("CALIBRATE_"
                                                   + str(self.numcam)
                                                   + ".jpg")
                    true_contador = 0.0
                    print "Guardo"
            else:
                false_contador += 0.001
                print "False: " + str(false_contador)
                if false_contador == 1.5:
                    cv.SaveImage("CALIBRATE_"
                                 + str(self.numcam)
                                 + ".jpg", frame)
                    frame_calibrate = cv.LoadImage("CALIBRATE_"
                                                   + str(self.numcam)
                                                   + ".jpg")
                    false_contador = 0.0

            k = cv.WaitKey(10)
            if k % 256 == 27:
                break

        cv.SaveImage("SUBTRACTION_" + str(self.numcam) + ".jpg", imagediff)
        cv.DestroyWindow('Subtraction ' + str(self.numcam))


"""
Programa principal especifico
"""


def ManagerModes(numcam, mode):
    cam = Scanner(numcam)
    if mode == 0:
        cam.ViewCam()
    elif mode == 1:
        cam.PhotoCalibrate()
    elif mode == 2:
        cam.ViewPhotoCalibrate()
    elif mode == 3:
        cam.PhotoCalibrate()
        cam.Sub_RGB()
    elif mode == 4:
        cam.Properties()
    elif mode == 5:
        cam.PhotoCalibrate()
        cam.Sub_HSV()

if __name__ == '__main__':

    print "SCANNER WEBCAM PROGRAM" + '\n'
    print "0 - View Cam" + '\n' + "1 - Photo Calibrate" + '\n'\
        + "2 - View Calibrate" + '\n' + "3 - Substraction" + '\n' + "4 - Exit"

    numcam = int(input("Cam: "))
    mode = int(input("Mode: "))
    if numcam == 1 or numcam == 2:
        cam = threading.Thread(target=ManagerModes, args=(numcam, mode))
        cam.start()
    if numcam == 3:
        for i in range(1, 3):
            print "cam " + str(i)
            cam = threading.Thread(target=ManagerModes, args=(i, 0))
            cam.start()
    """
    cam1 = threading.Thread(target=ManagerModes, args=(2, 3))
    cam2 = threading.Thread(target=ManagerModes, args=(1, 3))
    cam1.start()
    cam2.start()
    """
